; $Id$
;+
; NAME:
;   NUV_EXTRACT_EFF
;
; PURPOSE:
;   This procedure allows the user to select a series of files and to 
;   extract the count rates for each file for a specific wavelength
;   region depending on the grating and cenwave.
;   This tool is used to determine the change in the NUV grating efficiency
;   over time to monitor the bare Aluminium coating degradation.
;
; CALLING SEQUENCE:
;  NUV_EXTRACT_EFF, files =files
;
; OPTIONAL INPUTS:
;    PARENT:  Widget ID of the group leader for this widget
;
; MODIFICATION HISTORY:
;    Written by:   SBeland CU-Boulder/CASA, February 2005
;-
;***************************************************************************
PRO NUV_EXTRACT_EFF_EVENT, sEvent

    WIDGET_CONTROL, sEvent.id, GET_UVALUE=uval

    ; Handle KILL requests.
    IF TAG_NAMES(sEvent, /STRUCTURE_NAME) EQ 'WIDGET_KILL_REQUEST' THEN BEGIN
       ; Destroy the objects.
       WIDGET_CONTROL, sEvent.top, /DESTROY
       RETURN
    ENDIF ELSE IF TAG_NAMES(sEvent, /STRUCTURE_NAME) EQ 'WIDGET_TEXT_CH' THEN BEGIN
       RETURN
    ENDIF

    ; Get the state structure stored in the user value of the window
    WIDGET_CONTROL, GET_UVALUE=state, sEvent.top, /NO_COPY

    ; Handle other events.
    CASE uval OF
        'SELECT' : BEGIN
            ; bring up the file browser and add to list
            fileName = DIALOG_PICKFILE(GROUP=state.wBase, /MULTIPLE_FILES, /MUST_EXIST, $
                       PATH=state.initPath, GET_PATH=newpath, filter='*x1d*.fits')
            IF newpath NE state.initPath AND newpath NE '' THEN state.initPath = newpath

            ; If the user didn't cancel the standard file dialog
            IF STRLEN(fileName[0]) GT 0 THEN BEGIN
               ; first make sure that it is not already in the list
               IF (*state.listarr)[0] EQ "" THEN BEGIN
                  PTR_FREE, state.listarr
                  state.nlist = N_ELEMENTS(fileName)
                  state.listarr = PTR_NEW(fileName, /NO_COPY)
               ENDIF ELSE BEGIN
                  FOR i=0, N_ELEMENTS(fileName)-1 DO BEGIN
                     pos = WHERE(fileName[i] EQ (*state.listarr)[0:state.nlist-1])
                     IF pos[0] EQ -1 THEN BEGIN
                        *state.listarr = [*state.listarr, fileName[i]]
                        state.nlist = state.nlist + 1
                     ENDIF
                  ENDFOR
               ENDELSE
               WIDGET_CONTROL, state.wList, SET_VALUE=(*state.listarr)
               WIDGET_CONTROL, state.wProcessButton, SENSITIVE=1
               WIDGET_CONTROL, state.wClearButton, SENSITIVE=1
               WIDGET_CONTROL, state.wListLabel, $
                  SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
            ENDIF
        END

        'REMOVE' : BEGIN
            ; remove selected files from the list
            pos = WIDGET_INFO(state.wList, /LIST_SELECT)
            IF pos[0] NE -1 THEN BEGIN
               (*state.listarr)[pos] = ""
               state.nlist = state.nlist - N_ELEMENTS(pos)
               IF state.nlist EQ 0 THEN BEGIN
                  pos=0
                  PTR_FREE, state.listarr
                  state.listarr = PTR_NEW("")
                  WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
                  WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
                  WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
               ENDIF ELSE BEGIN
                  newlist = (*state.listarr)[WHERE(*state.listarr ne "")]
                  PTR_FREE, state.listarr
                  state.listarr = PTR_NEW(newlist, /NO_COPY)
               ENDELSE
               WIDGET_CONTROL, state.wList, SET_VALUE=(*state.listarr)
               WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
               WIDGET_CONTROL, state.wListLabel, $
                  SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
            ENDIF

          END

        'CLEAR' : BEGIN
            ; clear all the files in the list
            PTR_FREE, state.listarr
            state.listarr = PTR_NEW("")
            state.nlist = 0
            WIDGET_CONTROL, state.wList, SET_VALUE=['']
            WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
            WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
            WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
            WIDGET_CONTROL, state.wListLabel, $
               SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
          END

        'PROCESS' : BEGIN
             ; disable the buttons while processing
             WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
             WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
             WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
             WIDGET_CONTROL, state.wCancelButton, SENSITIVE=0
             WIDGET_CONTROL, state.wSelectButton, SENSITIVE=0

             ; get the list of keywords to extract
             keywords=['OPT_ELEM','CENWAVE','EXPTIME','COUNT','RATE']

             ; loop through every file 
             progress=0
             nfiles=state.nlist
             out_data=strarr(n_elements(keywords)+1,nfiles)
             nphotons=fltarr(nfiles)
             rates=fltarr(nfiles)
             ttime=fltarr(nfiles)

             ; initialize a progress meter
             p_meter = progress_meter(group_leader=sEvent.top, $
                                     TEXT='Extracting the data')

             WHILE state.nlist NE 0 DO BEGIN
                ; check if cancel button was pressed
                IF (*p_meter).cancelFlag THEN BEGIN
                   progress_Destroy, p_meter
                   Obj_Destroy, progressTimer
                   WIDGET_CONTROL, state.wCancelButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wSelectButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wClearButton, SENSITIVE=1
                   IF WIDGET_INFO(sEvent.top, /VALID_ID) THEN $
                      WIDGET_CONTROL, SET_UVALUE = state, sEvent.top, /NO_COPY
                   RETURN
                ENDIF

                ; store filename in data array
                fdecomp,(*state.listarr)[0],disk,dir,name,ext
                if nfiles le 1 then out_data[0]=name else out_data[0,progress]=name

                ; check for the type of input file
                data=mrdfits(((*state.listarr)[0]), 0, h0,/silent)
                data=mrdfits(((*state.listarr)[0]), 1, h1,/silent)
                header=[h0,h1]
                grating = strtrim(string(sxpar(header,'OPT_ELEM',/silent)),2)
                out_data[1,progress] = strtrim(string(grating),2)
                cenwave = strtrim(string(sxpar(header,'CENWAVE',/silent)),2)
                out_data[2,progress] = strtrim(string(cenwave),2)
                ttime[progress] = sxpar(header,'EXPTIME',/silent)
                out_data[3,progress] = strtrim(string(ttime[progress]),2)
                crval = sxpar(header,'CRVAL1',/silent)
                cd1_1 = sxpar(header,'cd1_1',/silent)

                ; figure out the wavelength region for this grating and cenwave
                nphotons[progress] = 0
                rates[progress]=0.0
                pos = where(state.wregion[*].grating eq grating and state.wregion[*].cenwave eq cenwave, count)
                if count gt 0 then begin
                   wr = state.wregion[pos].wr
                   wave = crval + findgen(1023)*cd1_1
                   p = where(wave ge wr[0] and wave le wr[1],count)
                   if count gt 0 then begin
                       nphotons[progress] = total(data.y[p])
                       rates[progress] = float(nphotons[progress]) / float(ttime[progress])
                   endif
                endif
                out_data[4,progress] = strtrim(string(nphotons[progress]),2)
                out_data[5,progress] = strtrim(string(rates[progress]),2)

                ; set the percentage done
                progress=progress+1
                progress_update, p_meter, FLOAT(progress)*100.0 / FLOAT(nfiles)

                ; remove the entry from the list
                (*state.listarr)[0] = ""
                state.nlist = state.nlist - 1
                IF state.nlist EQ 0 THEN BEGIN
                   PTR_FREE, state.listarr
                   state.listarr = PTR_NEW('')
                   WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
                   WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
                   WIDGET_CONTROL, state.wCancelButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wSelectButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
                   WIDGET_CONTROL, state.wList, SET_VALUE=['']
                ENDIF ELSE BEGIN
                   newlist = (*state.listarr)[WHERE(*state.listarr NE "")]
                   PTR_FREE, state.listarr
                   state.listarr = PTR_NEW(newlist, /NO_COPY)
                   WIDGET_CONTROL, state.wList, SET_VALUE=*state.listarr
                ENDELSE

             ENDWHILE

             ; get rid of meter
             progress_Destroy, p_meter

             IF state.nlist EQ 0 THEN BEGIN
                PTR_FREE, state.listarr
                state.listarr = PTR_NEW("")
                WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
                WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
                WIDGET_CONTROL, state.wCancelButton, SENSITIVE=1
                WIDGET_CONTROL, state.wSelectButton, SENSITIVE=1
                WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
                WIDGET_CONTROL, state.wList, SET_VALUE=['']
             ENDIF ELSE BEGIN
                newlist = (*state.listarr)[WHERE(*state.listarr ne "")]
                PTR_FREE, state.listarr
                state.listarr = PTR_NEW(newlist, /NO_COPY)
                WIDGET_CONTROL, state.wList, SET_VALUE=(*state.listarr)
                WIDGET_CONTROL, state.wSelectButton, SENSITIVE=1
                WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
                WIDGET_CONTROL, state.wCancelButton, SENSITIVE=1
             ENDELSE

             ; re-enable the buttons
             IF WIDGET_INFO(state.wList, /LIST_SELECT) EQ -1 THEN BEGIN
                WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0 
                WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
             ENDIF ELSE BEGIN
                WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=1
                WIDGET_CONTROL, state.wClearButton, SENSITIVE=1
             ENDELSE
             WIDGET_CONTROL, state.wListLabel, $
                SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
           
             ; return the result
             for i=0,n_elements(state.wregion)-1 do begin
                 p = where(out_data[2,*] eq state.wregion[i].cenwave,count)
                 if count eq 0 then continue
                 state.wregion[i].count = total(nphotons[p])
                 state.wregion[i].ttime = total(ttime[p])
                 state.wregion[i].rate = state.wregion[i].count / state.wregion[i].ttime
             endfor

             ;calculate the ratios and associated errors
             p1=where(state.wregion[*].cenwave eq 1986)
             p2=where(state.wregion[*].cenwave eq 2186)
             E1 = state.wregion[p1].rate / state.wregion[p2].rate
             S1 = E1 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             p1=where(state.wregion[*].cenwave eq 2010)
             p2=where(state.wregion[*].cenwave eq 2217)
             E2 = state.wregion[p1].rate / state.wregion[p2].rate
             S2 = E2 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             p1=where(state.wregion[*].cenwave eq 2390)
             p2=where(state.wregion[*].cenwave eq 2617)
             E3 = state.wregion[p1].rate / state.wregion[p2].rate
             S3 = E3 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             p1=where(state.wregion[*].cenwave eq 2410)
             p2=where(state.wregion[*].cenwave eq 2637)
             E4 = state.wregion[p1].rate / state.wregion[p2].rate
             S4 = E4 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             p1=where(state.wregion[*].cenwave eq 2010)
             p2=where(state.wregion[*].cenwave eq 3360)
             E5 = state.wregion[p1].rate / state.wregion[p2].rate
             S5 = E5 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             p1=where(state.wregion[*].cenwave eq 2217)
             p2=where(state.wregion[*].cenwave eq 3360)
             E6 = state.wregion[p1].rate / state.wregion[p2].rate
             S6 = E6 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             p1=where(state.wregion[*].cenwave eq 2637)
             p2=where(state.wregion[*].cenwave eq 2635)
             E7 = state.wregion[p1].rate / state.wregion[p2].rate
             S7 = E7 * sqrt(1.0/state.wregion[p1].count + 1.0/state.wregion[p2].count)

             print,''
             print,'Results'
             print,E1,S1,format='("W_1986 / W_2186:  E1=",F5.3,"  S1=",F5.3)'
             print,E2,S2,format='("W_2010 / W_2217:  E2=",F5.3,"  S2=",F5.3)'
             print,E3,S3,format='("W_2390 / W_2617:  E3=",F5.3,"  S3=",F5.3)'
             print,E4,S4,format='("W_2410 / W_2637:  E4=",F5.3,"  S4=",F5.3)'
             print,E5,S5,format='("W_2010 / W_3360:  E5=",F5.3,"  S5=",F5.3)'
             print,E6,S6,format='("W_2217 / W_3360:  E6=",F5.3,"  S6=",F5.3)'
             print,E7,S7,format='("W_2637 / W_2635:  E7=",F5.3,"  S7=",F5.3)'

             ; now display the data in a table
             show_data_table, out_data, col_labels=['Filename',keywords], format=format

             ; we can now exit this tool
             WIDGET_CONTROL, sEvent.top, /DESTROY
          END

        'CANCEL' : BEGIN
            ; quit without converting
            WIDGET_CONTROL, sEvent.top, /DESTROY
          END

        'LIST' : BEGIN
            ; a file was selected from the list
            pos = WIDGET_INFO(state.wList, /LIST_SELECT)
            IF pos[0] NE -1 THEN BEGIN
               WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=1
            ENDIF
        END

        ELSE: 
            ; ignore all other events
    ENDCASE

    ; Reset the windows user value to the updated state structure
    IF WIDGET_INFO(sEvent.top, /VALID_ID) THEN $
       WIDGET_CONTROL, SET_UVALUE = state, sEvent.top, /NO_COPY

END

;----------------------------------------------------------------------------
PRO NUV_EXTRACT_EFF, files=files, PARENT=parent

    ; Create the widgets.
    xsize = 130

    title = 'NUV_EXTRACT_EFF  '
    IF KEYWORD_SET(PARENT) THEN $
       wBase = WIDGET_BASE(GROUP_LEADER=parent, /ROW, $
                           TITLE=title, /TLB_KILL_REQUEST_EVENTS, TLB_FRAME_ATTR=1) $
    ELSE $
       wBase = WIDGET_BASE(/ROW, $
                           TITLE=title, /TLB_KILL_REQUEST_EVENTS, TLB_FRAME_ATTR=1)
 
    wGuiBase      = WIDGET_BASE(wBase, COL=2)
    wListBase     = WIDGET_BASE(wGuiBase, /COLUMN)
    wListLabel    = WIDGET_LABEL(wListBase, VALUE='Files to Process:  0', $
                    /ALIGN_LEFT, /DYNAMIC_RESIZE)
    wList         = WIDGET_LIST(wListBase, /MULTIPLE, XSIZE=30, YSIZE=11, UVALUE='LIST')
    wrow          = WIDGET_BASE(wListBase, /ROW)

    wButtons      = WIDGET_BASE(wGuiBase, /COLUMN, SPACE=53, YPAD=27)
    wTopButtons   = WIDGET_BASE(wButtons,/COLUMN, SPACE=10, frame=1)
    wSelectButton = WIDGET_BUTTON(wTopButtons, XSIZE=xsize, VALUE='Select', UVALUE='SELECT')
    wRemoveButton = WIDGET_BUTTON(wTopButtons, XSIZE=xsize, VALUE='Remove', UVALUE='REMOVE')
    wClearButton  = WIDGET_BUTTON(wTopButtons, XSIZE=xsize, VALUE='Clear', UVALUE='CLEAR')

    wBottomButtons= WIDGET_BASE(wButtons,/COLUMN, /BASE_ALIGN_BOTTOM, SPACE=10, frame=1)
    wProcessButton= WIDGET_BUTTON(wBottomButtons, XSIZE=xsize, VALUE='Process', UVALUE='PROCESS')
    wCancelButton = WIDGET_BUTTON(wBottomButtons, XSIZE=xsize, VALUE='Cancel', UVALUE='CANCEL')

    wregion = [$
        {grating:'G185M', cenwave:1986, wr:[2093.0,2105.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G185M', cenwave:2010, wr:[2126.0,2130.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G225M', cenwave:2186, wr:[2093.0,2105.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G225M', cenwave:2217, wr:[2126.0,2130.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G225M', cenwave:2390, wr:[2485.0,2500.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G225M', cenwave:2410, wr:[2503.0,2519.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G285M', cenwave:2617, wr:[2485.0,2500.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G285M', cenwave:2637, wr:[2503.0,2519.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G230L', cenwave:2635, wr:[2478.0,2533.0], ttime:0.0, count:0.0, rate:0.0}, $
        {grating:'G230L', cenwave:3360, wr:[2120.0,2140.0], ttime:0.0, count:0.0, rate:0.0} $
        ]

    state = { $
             wBase: wBase, $
             wList: wList, $
             wListLabel: wListLabel, $
             wSelectButton: wSelectButton, $
             wRemoveButton: wRemoveButton, $
             wClearButton: wClearButton, $
             wProcessButton: wProcessButton, $
             wCancelButton: wCancelButton, $
             listarr: PTR_NEW(""), $
             nlist: 0, $
             wregion:wregion, $
             initPath: cedar_getenv('CEDAR_REFERENCE_PATH') $
           }

    ; set the initial browsing path to the default raw dir set by environment
    path = CEDAR_GETENV('CEDAR_FITS_ROOT')
    IF path NE '' THEN state.initPath=path

    ; Save the state structure in the window's user value
    WIDGET_CONTROL, wBase, SET_UVALUE=state

    WIDGET_CONTROL, wRemoveButton, SENSITIVE=0
    WIDGET_CONTROL, wClearButton, SENSITIVE=0
    WIDGET_CONTROL, wProcessButton, SENSITIVE=0

    WIDGET_CONTROL, wBase, /REALIZE

    XMANAGER, 'NUV_EXTRACT_EFF', wBase, /NO_BLOCK

END
